__version__ = "1.0.1"
__description__ = "One-stop solution for AUTO testing."

# import firstly for monkey patch if needed
from pastor.ext.locust import main_locusts
from pastor.parser import parse_parameters as Parameters
from pastor.runner import Pastor
from pastor.testcase import Config, Step, RunRequest, RunTestCase, RunLocation

__all__ = [
    "__version__",
    "__description__",
    "Pastor",
    "Config",
    "Step",
    "RunLocation",
    "RunRequest",
    "RunTestCase",
    "Parameters",
    "Parameters2",
]
